#!/usr/bin/env python3
# -*- coding: utf-8 -*-
# Time-stamp: "2023-03-24 10:17:53 (ywatanabe)"

import random
import string

from datetime import datetime


def gen_ID(N=8):
    """Generates an ID like 'yyyy-mmdd-wwwwwwww'"""

    now = datetime.now()
    today_str = now.strftime("%Y-%m%d")
    randlst = [random.choice(string.ascii_letters + string.digits) for i in range(N)]
    rand_str = "".join(randlst)
    return today_str + "_" + rand_str



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
# Time-stamp: "2023-03-26 02:07:04 (ywatanabe)"
import os
import warnings

import openai
import speech_recognition as sr
import whisper
from gtts import gTTS


def s2t_unpaid(lpath_wav, language="EN"):
    """
    Speech to Text using a Google's product.

    Example:
        from scripts import audio, ml
        tmp_wav="/tmp/tmp.wav"
        audio.rec_wav_unlim(spath=tmp_wav)
        said = ml.s2t_unpaid(tmp_wav, language="EN")
        print(said)
        audio.play_audio(tmp_wav)
    """

    # print("\nText-to-speech ...\n")
    r = sr.Recognizer()
    with sr.AudioFile(lpath_wav) as source:
        audio = r.record(source)
    try:
        said_text = r.recognize_google(audio, language=language)
        # print(f"\nYou said:\n{said_text}\n")
        return said_text
    except Exception as e:
        print(e)


class ChatGPT(object):
    def __init__(
        self,
        system_setting="Please answer in a conversational manner as if you are my friend. "
        "I would be grateful if your response would be less than 20 words.",
    ):
        self.counter = 0
        openai.api_key = os.getenv("OPENAI_API_KEY") # "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"
        self.chat_history = []
        self.chat_history.append(
            {
                "role": "system",
                "content": system_setting,
            }
        )

    def __call__(self, text):
        self.counter += 1
        if text is None:
            text = ""
        try:
            self.chat_history.append({"role": "user", "content": text})
            response = openai.ChatCompletion.create(
                model="gpt-3.5-turbo",
                messages=self.chat_history,
                request_timeout=10,
                # max_tokens=30,
            )
            out_text = response["choices"][0]["message"]["content"]
            self.chat_history.append({"role": "assistant", "content": out_text})

            return out_text
        except Exception as e:
            print(e)
            out_text = "I could not catch you well."
            self.chat_history.append({"role": "assistant", "content": out_text})
            return out_text


# def t2t_chatGPT(text):
#     """
#     Text to text using ChatGPT by OpenAI.

#     Example:
#         from scripts import ml
#         out = ml.t2t_chatGPT("Please help me learn English.")
#         print(out)
#     """
#     openai.api_key = "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"
#     time_lim_sec = 10

#     try:
#         add_text = " Please answer in 20 words or less as if you are my friend."
#         response = openai.ChatCompletion.create(
#             model="gpt-3.5-turbo",
#             messages=[
#                 {"role": "system", "content": text + add_text},
#             ],
#             request_timeout=time_lim_sec,
#             max_tokens=30,
#         )
#         responded_text = response["choices"][0]["message"]["content"]
#         return responded_text
#     except Exception as e:
#         print(e)
#         return "I could not catch you well."


def t2s(text, spath="/tmp/t2s.mp3", print_save=False):
    """
    Text to Speech using a Google's product.

    Example:
        from scripts import audio, ml
        # import utils
        text = "This is the test of my t2s function."
        spath="/tmp/t2s.mp3"
        ml.t2s(text, spath=spath)
        audio.play_audio(spath)
    """
    speech = gTTS(text=text, lang="en", slow=False)
    speech.save(spath)
    if print_save:
        print(f"Saved to: {spath}")


class Whisper(object):
    """
    Speach to text using Whisper by OpenAI.

    from scripts import audio, ml

    spath = "/tmp/test.wav"
    audio.rec_wav_unlim(spath=spath, fs=44100)

    mywhisper = Whisper()
    mywhisper(spath)
    """

    def __init__(
        self,
    ):
        self.model = whisper.load_model("base")

    def __call__(self, lpath):
        with warnings.catch_warnings():
            warnings.simplefilter("ignore", UserWarning)
            result = self.model.transcribe(lpath)
        return result["text"]



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
# Time-stamp: "2023-03-26 02:06:06 (ywatanabe)"

import audio2numpy as a2n
import numpy as np
import soundcard as sc
from scipy.io import wavfile
import threading
import readchar


class ThreadWithReturnValue(threading.Thread):
    """
    Example:
        t = ThreadWithReturnValue(
            target=func, args=(,), kwargs={key: val}
        )
        t.start()
        out = t.join()

    """

    def __init__(
        self, group=None, target=None, name=None, args=(), kwargs={}, Verbose=None
    ):
        threading.Thread.__init__(self, group, target, name, args, kwargs)
        self._return = None

    def run(self):
        if self._target is not None:
            self._return = self._target(*self._args, **self._kwargs)

    def join(self, *args):
        threading.Thread.join(self, *args)
        return self._return


def load_wav_as_arr(lpath):
    fs, arr = wavfile.read(lpath)
    return fs, arr


def save_arr_as_wav(arr, spath, fs=44100, dtype=np.int16, does_print=False):
    arr = arr.astype(np.float64)
    # with warnings.catch_warnings():
    #     warnings.simplefilter("ignore", RuntimeWarning)
    arr /= arr.max()
    amp = np.iinfo(dtype).max
    arr *= amp
    wavfile.write(spath, fs, arr.astype(dtype))
    if does_print:
        print(f"Saved to: {spath}")


def play_arr(arr, fs=44100):
    """
    Example:
        fs = 44100
        arr = rec_arr(rec_sec=5, fs=fs)
        play_arr(arr, fs=fs)
    """
    # speakers = sc.all_speakers()
    default_speaker = sc.default_speaker()
    # print(default_speaker)
    default_speaker.play(arr / np.max(arr), samplerate=fs)


def play_audio(lpath, fs=None):
    """
    Example:
        fs = 44100
        spath="/tmp/test_rec_wav_unlim.wav"
        rec_wav_unlim(spath=spath, fs=fs)
        play_audio(spath)
    """
    arr, _fs = a2n.audio_from_file(lpath)
    fs = fs if fs is not None else _fs
    play_arr(arr, fs=fs)


def play_buzzer():
    play_audio("./data/buzzer.wav")


def rec_as_arr(rec_sec=5, fs=44100):
    """
    Example:
        arr = rec_as_arr(rec_sec=5, fs=44100)
        play_arr(arr, fs=fs)
    """
    # mics = sc.all_microphones()
    default_mic = sc.default_microphone()
    # print(default_mic)
    play_buzzer()
    arr = default_mic.record(samplerate=fs, numframes=fs * rec_sec)
    return arr


def rec_as_arr_unlim(fs=44100, print_press_q_message=True):
    """
    Records microphone input unlimitedly.

    Example:
        fs = 44100
        arr_unlim = rec_as_arr_unlim(fs=fs)
        play_arr(arr_unlim, fs=fs)
    """

    def _wait_space():
        pressed_key = None
        while pressed_key != " ":
            pressed_key = readchar.readchar()
        event.set()

    def _rec_as_arr_unlim():
        default_mic = sc.default_microphone()
        chunk_size = 2**14

        arr_unlim = []
        play_buzzer()

        with default_mic.recorder(samplerate=fs) as mic:
            while not event.is_set():
                data = mic.record(numframes=chunk_size)
                arr_unlim.append(data)
        return np.vstack(arr_unlim).astype(np.float64)

    if print_press_q_message:
        print("\nSPACE: Stop recording.\nSPACE -> Ctrl + C: Stop the session.\n")

    event = threading.Event()
    t1 = ThreadWithReturnValue(target=_wait_space)
    t2 = ThreadWithReturnValue(target=_rec_as_arr_unlim)

    t1.start()
    t2.start()

    return t2.join()


def rec_as_wav(spath="/tmp/test_rec_as_wav_unlim.wav", rec_sec=5, fs=44100):
    """
    Example:
        rec_sec = 5
        fs = 44100
        spath="/tmp/test_rec_as_wav.wav"
        arr = rec_as_wav(spath=spath, rec_sec=rec_sec, fs=fs)
    """
    play_buzzer()
    arr = rec_as_arr(rec_sec=rec_sec, fs=fs)
    save_arr_as_wav(arr, spath, fs=fs)


def rec_as_wav_unlim(
    spath="/tmp/test_rec_as_wav_unlim.wav", fs=44100, print_press_q_message=True
):
    """
    Example:
        fs = 44100
        spath="/tmp/test_rec_as_wav_unlim.wav"
        rec_as_wav_unlim(spath=spath, fs=fs)
    """
    arr = rec_as_arr_unlim(fs=fs, print_press_q_message=print_press_q_message)
    save_arr_as_wav(arr, spath=spath, fs=fs)


if __name__ == "__main__":
    fs = 44100
    arr_unlim = rec_as_arr_unlim(fs=fs)
    play_arr(arr_unlim, fs=fs)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
# Time-stamp: "2023-03-26 02:01:58 (ywatanabe)"

import sys
import time

sys.path.append(".")
# from scripts import audio, ml, utils
from vChatGPT.scripts import audio, ml, utils


def s2s(chatgpt):
    ID = utils.gen_ID()

    # Records a microphone input
    spath_in_wav = f"/tmp/input-{ID}.wav"
    audio.rec_as_wav_unlim(
        spath=spath_in_wav,
        print_press_q_message=True if chatgpt.counter == 0 else False,
    )
    time.sleep(0.1)

    # Speech to text
    # if "mywhisper" not in globals():
    #     mywhisper = utils.Whisper()
    # said = mywhisper(spath_in_wav)
    said = ml.s2t_unpaid(spath_in_wav, language="EN")
    print(said)

    # Text to text (using ChatGPT)
    # gpt_out = ml.t2t_chatGPT(said)
    gpt_out = chatgpt(said)
    print(gpt_out)
    print()

    # Text to speech
    spath_out_mp3 = f"/tmp/output-{ID}.mp3"
    ml.t2s(gpt_out, spath=spath_out_mp3, print_save=False)

    # Plays the ChatGPT's output
    audio.play_audio(spath_out_mp3)

def main():
    chatgpt = ml.ChatGPT()
    counter = 0
    while True:
        s2s(chatgpt)
        counter += 1
    
if __name__ == "__main__":
    main()



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
# Time-stamp: "2023-03-26 02:20:28 (ywatanabe)"

from .scripts.main import main

__copyright__ = "Copyright (C) 2023 Yusuke Watanabe"
__version__ = "0.11"
__license__ = "GPL3.0"
__author__ = "ywatanabe1989"
__author_email__ = "ywata1989@gmail.com"
__url__ = "https://github.com/ywatanabe1989/vChatGPT"

